/*
 * dproc.c - AIX process access functions for lsof
 */


/*
 * Copyright 1994 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: dproc.c,v 1.1 94/04/13 08:37:29 abe Exp $";
#endif


#include "lsof.h"


_PROTOTYPE(static void get_kernel_access,(void));
_PROTOTYPE(static struct le *getle,(long a, struct user *u, long sid));
_PROTOTYPE(static void process_text,(long sid));


/*
 * Local static values
 */

static int Np = 0;			/* number of processes */
static struct procinfo *P = NULL;	/* the process table */
static struct user *U;			/* user structure */


/*
 * gather_proc_info() - gather process information
 */

void
gather_proc_info()
{
	int i;
	short pss, sf;
	struct user us;
/*
 * Loop through processes.
 */
	for (U = &us; Np > 0; Np--, P++) {
		if (P->p_stat == 0 || P->p_stat == SZOMB)
			continue;
		if (is_proc_excl(P->p_pid, (uid_t)P->p_uid, &pss, &sf))
			continue;
		if (getuser(P, PROCSIZE, U, U_SIZE) != 0)
			continue;
	/*
	 * Allocate a local process structure and start filling it.
	 */
		alloc_lproc(P->p_pid, (uid_t)P->p_uid, U->u_comm, pss, sf);
		Plf = NULL;
	/*
	 * Save current working directory information.
	 */
		if (U->u_cdir) {
			alloc_lfile(CWD, -1);
			process_node((caddr_t)U->u_cdir);
			if (Lf->sf)
				link_lfile();
		}
	/*
	 * Save root directory information.
	 */
		if (U->u_rdir) {
			alloc_lfile(RTD, -1);
			process_node((caddr_t)U->u_rdir);
			if (Lf->sf)
				link_lfile();
		}
	/*
	 * Save parent directory information.
	 */
		if (U->u_pdir) {
			alloc_lfile("  pd", -1);
			process_node((caddr_t)U->u_pdir);
			if (Lf->sf)
				link_lfile();
		}
	/*
	 * Save information on text files.
	 */
		process_text(P->pi_adspace);
	/*
	 * Save information on file descriptors.
	 */
		for (i = 0; i < U->u_maxofile; i++) {
			if (U->u_ufd[i].fp) {
				alloc_lfile(NULL, i);
				process_file((KA_T)U->u_ufd[i].fp);
				if (Lf->sf)
					link_lfile();
			}
		}
	/*
	 * Examine results.
	 */
		if (examine_lproc())
			return;
	}
}


/*
 * get_kernel_access() - get access to kernel memory
 */

static void
get_kernel_access()
{
	if ((Kmem = open(Core ? Core : KMEM, O_RDONLY, 0)) < 0) {
		(void) fprintf(stderr, "%s: can't open %s: %s\n", Pn,
			Core ? Core : KMEM, strerror(errno));
		exit(1);
	}
	if ((Mem = open("/dev/mem", O_RDONLY, 0)) < 0) {
		(void) fprintf(stderr, "%s: can't open /dev/mem: %s\n",
			Pn, strerror(errno));
		exit(1);
	}
	if (knlist(&Nl, 1, sizeof(struct nlist)) ||  Nl[0].n_value == NULL) {
		(void) fprintf(stderr, "%s: can't get user area address\n");
		exit(1);
	}
}


/*
 * getle() - get loader entry structure
 */

static struct le *
getle(a, u, sid)
	long a;				/* loader entry kernel address */
	struct user *u;			/* load user structure address */
	long sid;			/* user structure sedment ID */
{
	static struct le le;

	if (a < Nl[0].n_value)
		return(NULL);
	if (((char *)a + sizeof(le)) <= ((char *)Nl[0].n_value + U_SIZE))
		return((struct le *)((char *)u + a - Nl[0].n_value));
	if (kreadx((off_t)a, (char *)&le, sizeof(le), sid) == 0)
		return(&le);
	return(NULL);
}


/*
 * initialize() - perform all initialization
 */

void
initialize()
{
	get_kernel_access();
/*
 * Read the process table.
 */
	if ((P = (struct procinfo *)malloc((size_t)PROCSIZE)) == NULL) {
		(void) fprintf(stderr,
			"%s: can't allocate space for 1 procinfo\n", Pn);
		exit(1);
	}
	while (((Np = getproc(P, Np, PROCSIZE)) == -1) && errno == ENOSPC) {
		Np = P->p_pid + 10;
		if ((P = (struct procinfo *)realloc((MALLOC_P *)P,
		    (size_t) (Np * PROCSIZE))) == NULL) {
			(void) fprintf(stderr,
				"%s: no space for %ld procinfo's\n", Pn, Np);
			exit(1);
		}
	}
	iuidcache(Np);
}


/*
 * kread() - read from kernel memory
 */

int
kread(addr, buf, len)
	KA_T addr;			/* kernel memory address */
	char *buf;			/* buffer to receive data */
	READLEN_T len;			/* length to read */
{
	int br;

	if (lseek(Kmem, addr, L_SET) == (off_t)-1L)
		return(-1);
	br = read(Kmem, buf, len);
	return((br == len) ? 0 : 1);
}


/*
 * kreadx() - read kernel segmented memory
 */

int
kreadx(addr, buf, len, sid)
	off_t addr;			/* kernel address */
	char *buf;			/* destination buffer */
	int len;			/* length */
	long sid;			/* segment ID */
{
	int br;

	if (lseek(Mem, addr, L_SET) == (off_t)-1)
		return(1);
	br = readx(Mem, buf, len, sid);
	return (br == len ? 0 : 1);
}


/*
 * process_text() - process text file information
 */

static void
process_text(sid)
	long sid;			/* user area segment ID */
{
	static struct file **f = NULL;
	char fd[8];
	int i, j, n;
	struct le *le;
	long ll;
	static MALLOC_S nf = 0;
/*
 * Display information on the exec'd entry.
 */
	if ((le = getle(U->u_loader[1], U, sid)) != NULL && le->fp) {
		alloc_lfile(" txt", -1);
		process_file((struct file *)le->fp);
		if (Lf->sf)
			link_lfile();
	}
/*
 * Display the loader list.
 */
	for (i = n = 0, ll = U->u_loader[0]; ll; ll = (long)le->next) {
		(void) sprintf(fd, " L%02d", i);
		alloc_lfile(fd, -1);
		if ((le = getle(ll, U, sid)) == NULL) {
			(void) sprintf(Namech, "can't read loader entry at %#x",
				ll);
			enter_nm(Namech);
			link_lfile();
			return;
		}
	/*
	 * Skip the exec'd file and entries with no file pointers.
	 */
		if (ll == U->u_loader[1] || ! le->fp)
			continue;
	/*
	 * Skip entries with a file pointer that has already been
	 * displayed.  Record new, unique file pointers.
	 */
		for (j = 0; j < n; j++) {
			if (f[j] == le->fp)
				break;
		}
		if (j < n)
			continue;
		if (f == NULL) {
			if ((f = (struct file **)malloc((MALLOC_S)
				(sizeof(struct file *) * 10)))
			== NULL) {
			    (void) fprintf(stderr,
				"%s: no space for text file pointers\n", Pn);
			    exit(1);
			}
			nf = 10;
		} else if (n >= nf) {
			nf += 10;
			if ((f = (struct file **)realloc((MALLOC_P *)f,
				(MALLOC_S)(nf * sizeof(struct file *))))
			== NULL) {
			    (void) fprintf(stderr,
				"%s: no more space for text file pointers\n",
				Pn);
			    exit(1);
			}
		}
		f[n++] = le->fp;
	/*
	 * Save the loader entry.
	 */
		process_file((struct file *)le->fp);
		if (Lf->sf) {
			link_lfile();
			i++;
		}
	}
}
