/*
 * dnode.c - BSDI node functions for lsof
 */


/*
 * Copyright 1994 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: dnode.c,v 1.4 94/05/06 16:01:10 abe Exp $";
#endif


#include "lsof.h"


/*
 * process_node() - process vnode
 */

void
process_node(va)
	caddr_t va;			/* vnode kernel space address */
{

	dev_t dev;
	char dev_ch[32];
	struct inode *i = NULL;
	struct vfs *la = NULL;
	struct lockf lf;
	int lt;
	struct mfsnode *m = NULL;
	struct nfsnode *n = NULL;
	char *ty;
	enum vtype type;
	struct vnode *v, vb;
	struct l_vfs *vfs;
/*
 * Read the vnode.
 */
	if ( ! va) {
		enter_nm("no vnode address");
		return;
	}
	v = &vb;
	if (readvnode((caddr_t)va, v)) {
                enter_nm(Namech);
                return;
        }

/*
 * Get the vnode type.
 */
	if (v->v_mount == NULL || (vfs = readvfs(v->v_mount)) == NULL) {
		(void) sprintf(Namech, "bad mount for %#x at %#x", v,
			v->v_mount);
		enter_nm(Namech);
		return;
	}
	switch (vfs->type) {
	case MOUNT_NFS:
		Ntype = N_NFS;
		break;
	}
	if (Ntype == N_REGLR) {
		switch (v->v_type) {
		case VFIFO:
			Ntype = N_FIFO;
			break;
		}
	}
/*
 * Define the specific BSD386 node pointer.
 */
	switch (v->v_tag) {
	case VT_MFS:
		m = (struct mfsnode *)v->v_data;
		break;
	case VT_NFS:
		n = (struct nfsnode *)v->v_data;
		break;
	case VT_UFS:
		i = (struct inode *)v->v_data;
		if (i->i_lockf != NULL
		&&  kread((KA_T)i->i_lockf, (char *)&lf, sizeof(lf)) == 0) {

		/*
		 * Determine the lock state.
		 */
			if (lf.lf_start == (off_t)0 && lf.lf_end == (off_t)-1)
				lt = 1;
			else
				lt = 0;
			if (lf.lf_type == F_RDLCK)
				Lf->lock = lt ? 'R' : 'r';
			else if (lf.lf_type == F_WRLCK)
				Lf->lock = lt ? 'W' : 'w';
		}
		break;
	}
/*
 * Get device and type for printing.
 */
	type = v->v_type;
	if (n != NULL)
		dev = n->n_vattr.va_fsid;
	else if (i != NULL)
		dev = (type == VCHR) ? i->i_rdev : i->i_dev;
/*
 * Obtain the inode number.
 */
	if (i != NULL)
		Lf->inode = (long)i->i_number;
	else if (n != NULL)
		Lf->inode = n->n_vattr.va_fileid;
/*
 * Obtain the file size.
 */
	if (Foffset)
		Lf->off_def = 1;
	else {
		switch (Ntype) {
		case N_FIFO:
			if (!Fsize)
				Lf->off_def = 1;
			break;
		case N_NFS:
			if (n != NULL) {
				Lf->sz = (off_t)n->n_vattr.va_size;
				Lf->sz_def = 1;
			}
			break;
		case N_REGLR:
			if (type == VREG || type == VDIR) {
				if (i != NULL) {
					Lf->sz = i->i_size;
					Lf->sz_def = 1;
				} else if (m != NULL) {
					Lf->sz = m->mfs_size;
					Lf->sz_def = 1;
				}
			}
			else if (type == VCHR && !Fsize)
				Lf->off_def = 1;
			break;
		}
	}
/*
 * Record an NFS file selection.
 */
	if (Ntype == N_NFS && Fnfs)
		Lf->sf |= SELNFS;
/*
 * Save the file system names.
 */
	if (vfs) {
		Lf->fsdir = vfs->dir;
		Lf->fsdev = vfs->fsname;
	}
/*
 * Format the vnode type, and possibly the device name.
 */
	switch (type) {

	case VNON:
		ty ="VNON";
		break;
	case VREG:
	case VDIR:
		ty = (type == VREG) ? "VREG" : "VDIR";
		Lf->dev_def = 1;
		Lf->dev = dev;
		break;
	case VBLK:
		ty = "VBLK";
		break;
	case VCHR:
		Lf->dev = dev;
		Lf->dev_def = Lf->is_chr_dev = 1;
		ty = "VCHR";
		break;
	case VLNK:
		ty = "VLNK";
		break;

#if	defined(VSOCK)
	case VSOCK:
		ty = "SOCK";
		break;
#endif

	case VBAD:
		ty = "VBAD";
		break;
	case VFIFO:
		Lf->dev = dev;
		Lf->dev_def = 1;
		ty = "FIFO";
		break;
	default:
		if (type > 9999)
			(void) sprintf(Lf->type, "*%03d", type % 1000);
		else
			(void) sprintf(Lf->type, "%4d", type);
		(void) strcpy(Namech, "unknown type");
		ty = NULL;
	}
	if (ty)
		(void) strcpy(Lf->type, ty);
/*
 * Handle some special cases:
 *
 * 	ioctl(fd, TIOCNOTTY) files;
 *	memory node files;
 *	/proc files.
 */

	if (type == VBAD)
		(void) strcpy(Namech, "(dead)");
	else if (m != NULL) {
		Lf->dev_def = 0;
		(void) sprintf(Namech, "%#x", m->mfs_baseoff);
		(void) strcpy(dev_ch, "    memory");
		enter_dev_ch(dev_ch);
	}
/*
 * If this is a VCHR file and it's missing an inode number, try to
 * supply one.
 */
	if (Lf->inode == -1L && type == VCHR && Lf->dev_def)
		find_ch_ino();
/*
 * Test for specified file.
 */
	if (Sfile && is_file_named(NULL, type))
		Lf->sf |= SELNM;
/*
 * Enter name characters.
 */
	if (Namech[0])
		enter_nm(Namech);
}
