/*
 * dproc.c - HP-UX process access functions for lsof
 */


/*
 * Copyright 1994 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: dproc.c,v 1.1 94/04/14 07:34:24 abe Exp $";
#endif

#include "lsof.h"


/*
 * Local static values
 */

static KA_T Kp;				/* kernel's process table address */
static int Np;				/* number of kernel processes */

#if	defined(HPUX8)
static MALLOC_S Nv = 0;			/* number of entries in vnode cache */
static struct vnode **Vp = NULL;	/* vnode cache */
#endif


_PROTOTYPE(static void get_kernel_access,(void));

#if	defined(HPUX8)
_PROTOTYPE(static void process_text,(struct vas *vasp));
#endif


/*
 * gather_proc_info() -- gather process information
 */

void
gather_proc_info()
{
	int err, i, j;

#if	defined(HPUX8)
	char *c, *s;
	struct ofile_t *ofp;
	struct ofile_t oft;
	struct pst_status ps;
	struct user us;
#else
	int k;
	long sw;
	char us[U_SIZE];	/* must read HP-UX SWAP in DEV_BSIZE chunks */

# if	defined(hp9000s300)
	struct pte pte1, pte2;
	KA_T pte_off, pte_addr;
# endif
#endif

	struct proc *p;
	KA_T pa;
	struct proc pbuf;
	short pss, sf;
	int px;
	struct user *u;
/*
 * Examine proc structures and their associated information.
 */

#if	defined(HPUX8)
	for (p = &pbuf, px = 0, u = &us; px < Np; px++)
#else
	for (p = &pbuf, px = 0, u = (struct user *)us; px < np; px++)
#endif
	{
		pa = Kp + (KA_T)(px * sizeof(struct proc));
		if (kread(pa, (char *)&pbuf, sizeof(pbuf)))
			continue;
		if (p->p_stat == 0 || p->p_stat == SZOMB)
			continue;
	/*
	 * See if process is excluded.
	 */
		if (is_proc_excl(p->p_pid, (uid_t)p->p_uid, &pss, &sf))
			continue;
	/*
	 * Read the user area.
	 */

#if	defined(HPUX8)
/*
 * Use the undocumented HP-UX 8.x pstat() syscall to read process status.
 */
	(void) zeromem(u, U_SIZE);
	if (pstat(PSTAT_PROC, &ps, sizeof(ps), 0, p->p_pid) != 1) {
		(void) fprintf(stderr, "%s: can't pstat process %d: %s\n",
			Pn, p->p_pid, strerror(errno));
		continue;
	}
/*
 * Scan the pst_cmd command buffer and skip to the last component of the
 * first path name.  Also skip any leading `-', signifying a login shell.
 * Copy the result to u_comm[].
 */
	c = ps.pst_cmd;
	ps.pst_cmd[PST_CLEN - 1] = '\0';	/* paranoia */
	if (*c == '/' || *c == '.') {
		for (s = c; *s && *s != ' '; s++) {
			if (*s == '/')
				c = s + 1;
		}
	}
	if (*c == '-')
		c++;
	for (i = 0; i < MAXCOMLEN; i++) {
		if (*c == '\0' || *c == ' ' || *c == '/')
			break;
		u->u_comm[i] = *c++;
	}
	u->u_comm[i] = '\0';
#else	/* !HPUX8 */

/*
 * Read the user area from the swap file or memory.
 */
	if ((p->p_flag & SLOAD) == 0) {

	/*
	 * If the process is not loaded, read the user area from the swap file.
	 */
		if (Swap < 0)
			continue;
		sw = (long)p->p_swaddr;

# if	defined(hp9000s800)
		sw += (long)ctod(btoc(STACKSIZE * NBPG));
# endif

		if (lseek(Swap, (off_t)dtob(sw), L_SET) == (off_t)-1
		||  read(Swap, u, U_SIZE) != U_SIZE)
			continue;
	} else {

	/*
	 * Read the user area via the page table.
	 */

# if	defined(hp9000s300)
		pte_off = (KA_T) &Usrptmap[btokmx(p->p_p0br) + p->p_szpt - 1];
		if (kread(pte_off, (char *)&pte1, sizeof(pte1)))
			continue;
		pte_addr = (KA_T)(ctob(pte1.pg_pfnum + 1)
			 - ((UPAGES + FLOAT) * sizeof(pte2)));
		if (mread(pte_addr, (char *)&pte2, sizeof(pte2)))
			continue;
		if (mread((KA_T)ctob(pte2.pg_pfnum), (char *)u,
		    sizeof(struct user)))
			continue;
# endif

# if	defined(hp9000s800)
		if (kread((KA_T)uvadd((struct proc *)pa), (char *)u,
		    sizeof(struct user)))
			continue;
	}
# endif
#endif	/* HPUX8 */

	/*
	 * Allocate a local process structure.
	 */
		alloc_lproc(p->p_pid, (uid_t)p->p_uid, u->u_comm, pss, sf);
		Plf = NULL;
	/*
	 * Save current working directory information.
	 */
		if (CURDIR) {
			alloc_lfile(CWD, -1);
			process_node((caddr_t)CURDIR);
			if (Lf->sf)
				link_lfile();
		}
	/*
	 * Save root directory information.
	 */
		if (ROOTDIR) {
			alloc_lfile(RTD, -1);
			process_node((caddr_t)ROOTDIR);
			if (Lf->sf)
				link_lfile();
		}

#if	defined(HPUX8)
	/*
	 * Print information on the text file.
	 */
		if (p->p_vas)
			process_text(p->p_vas);
#endif	/* HPUX8 */

	/*
	 * Loop through user's files.
	 */

#if	defined(HPUX8)
		for (i = 0, j = SFDCHUNK; i < p->p_maxof; i++)
#else
		for (i = j = k = 0;; i++)
#endif

		{

#if	defined(HPUX8)
			if (j >= SFDCHUNK) {
			    if (kread((KA_T)p->p_ofilep, (char *)&ofp,
				       sizeof(ofp)))
			    {
				(void) fprintf(stderr,
				    "%s: PID %d, can't read ofilep from %#x\n",
				    Pn, p->p_pid, ofp);
				exit(1);
			    }
			    if (ofp == NULL)
				break;
			    if (kread((KA_T)ofp, (char *)&oft, sizeof(oft))) {
				(void) fprintf(stderr,
				    "%s: PID %d, can't read ofile_t from %#x\n",
				    Pn, p->p_pid, ofp);
				exit(1);
			    }
			    j = 0;
			    p->p_ofilep++;
			}
			j++;
			if (oft.ofile[j - 1])
#else	/* !HPUX8 */
			if (j >= SFDCHUNK) {

			/*
			 * Get next file pointer "chunk".
			 */
			    while (++k < NFDCHUNKS && u->u_ofilep[k] == NULL)
				;
			    if (k >= NFDCHUNKS)
				break;
			    if (kread((KA_T)u->u_ofilep[k],
				      (char *)&u->u_ofile,
				      sizeof(struct ofile_t)))
				break;
			    j = 0;
			}
			j++;
			if (u->u_ofile.ofile[j - 1])
#endif	/* HPUX8 */

		/*
		 * Process the file pointer.
		 */

			{
				alloc_lfile(NULL, i);
				process_file(FILEPTR);
				if (Lf->sf)
					link_lfile();
			}
		}
	/*
	 * Examine results.
	 */
		if (examine_lproc())
		return;
	}
}


/*
 * get_kernel_access() - access the required information in the kernel
 */

static void
get_kernel_access()
{

/*
 * Open kernel memory access.
 */
	if ((Kmem = open(Core ? Core : KMEM, O_RDONLY, 0)) < 0) {
		(void) fprintf(stderr, "%s: can't open %s: %s\n", Pn,
			Core ? Core : KMEM, strerror(errno));
		exit(1);
	}

#if	!defined(HPUX8)
	if ((Mem = open("/dev/mem", O_RDONLY, 0)) < 0) {
		(void) fprintf(stderr, "%s: can't open /dev/mem: %s\n",
			Pn, strerror(errno));
		err = 1;
	}
	if (Core == NULL || strcmp(Core, KMEM) == 0) {
		if ((Swap = open(SWAP, O_RDONLY, 0)) < 0) {
			(void) fprintf(stderr, "%s: %s: %s\n",
				Pn, SWAP, strerror(errno));
			err = 1;
		}
	}
#endif

/*
 * Access kernel symbols.
 */
	if (nlist(Nmlst ? Nmlst : N_UNIX, Nl) < 0) {
                (void) fprintf(stderr, "%s: can't read namelist from %s\n",
			Pn, Nmlst ? Nmlst : N_UNIX);
                exit(1);
	}
	if (Nl[X_PROC].n_value == NULL
	||  kread((KA_T)Nl[X_PROC].n_value, (char *)&Kp, sizeof(Kp))
	||  Nl[X_NPROC].n_value == NULL
	||  kread((KA_T)Nl[X_NPROC].n_value, (char *)&Np, sizeof(Np))
	||  Kp == NULL || Np == 0) {
		(void) fprintf(stderr, "%s: can't read proc table info\n",
			Pn);
		exit(1);
	}
	Vnfops = (struct fileops *) Nl[X_VN_FOPS].n_value;

#if	!defined(HPUX8) && defined(hp9000s300)
	if ((Usrptmap = (struct pte *)Nl[X_USRPTMAP].n_value) == NULL) {
		(void) fprintf(stderr, "%s: can't get kernel's Usrptmap\n", Pn);
		exit(1);
	}
	if ((usrpt = (struct pte *)Nl[X_USRPT].n_value) == NULL) {
		(void) fprintf(stderr, "%s: can't get kernel's usrpt\n", Pn);
		exit(1);
	}
#endif

#if	!defined(HPUX8) && defined(hp9000s800)
	proc = (struct proc *)Kp;
	if ((ubase = (struct user *)Nl[X_UBASE].n_value) == NULL) {
		(void) fprintf(stderr, "%s: can't get kernel's ubase\n", Pn);
		exit(1);
	}
	if (Nl[X_NPIDS].n_value == NULL
	||  kread((KA_T)Nl[X_NPIDS].n_value, (char *)&npids, sizeof(npids))) {
		(void) fprintf(stderr, "%s: can't get kernel's npids\n", Pn);
		exit(1);
	}
#endif

}


/*
 * initialize() - perform all initialization
 */

void
initialize()
{
	get_kernel_access();
	(void) iuidcache(Np);
}


/*
 * kread() - read from kernel memory
 */

int
kread(addr, buf, len)
	KA_T addr;			/* kernel memory address */
	char *buf;			/* buffer to receive data */
	READLEN_T len;			/* length to read */
{
	int br;

	if (lseek(Kmem, addr, L_SET) == (off_t)-1L)
		return(-1);
	br = read(Kmem, buf, len);
	return((br == len) ? 0 : 1);
}


#if	!defined(HPUX8)
/*
 * mread() -- read from /dev/mem
 */

static int
mread(addr, buf, len)
	KA_T addr;			/* /dev/mem address */
	char *buf;			/* buffer to receive data */
	READLEN_T len;			/* length to read */
{
	int br;

	if (lseek(Mem, addr, L_SET) == (off_t)-1L)
		return(1);
	br = read(Mem, buf, len);
	return((br == len) ? 0 : 1);
}
#endif



#if	defined(HPUX8)
/*
 * process_text() - process text access information
 */

static void
process_text(vasp)
	struct vas *vasp;		/* virtual address space pointer */
{
	char fd[FDLEN];
	int i, j, lm;
	struct pregion p, *prp;
	struct region r;
	struct vas v;
	struct vnode *va;
/*
 * Read virtual address space pointer.
 */
	if (kread((KA_T)vasp, &v, sizeof(v)))
		return;
/*
 * Follow the virtual address space pregion structure chain.
 */
	for (i = lm = 0, prp = v.va_next;
	     prp != (struct pregion *)vasp;
	     prp = p.p_next, lm++)
	{

	/*
	 * Avoid infinite loop.
	 */
		if (lm > 1000) {
		    if ( ! Fterse)
			(void) fprintf(stderr,
			    "%s: too many virtual address regions for PID %d\n",
			    Pn, Lp->pid);
		    return;
		}
	/*
	 * Read the pregion and region.
	 */
		if (kread((KA_T)prp, (char *)&p, sizeof(p)))
			return;
		if (kread((KA_T)p.p_reg, (char *)&r, sizeof(r)))
			return;
	/*
	 * Skip file entries with no file pointers.
	 */
		if ((va = r.r_fstore) == NULL)
			continue;
	/*
	 * Skip entries whose vnodes have already been displayed.
	 *
	 *  Record new, unique vnode pointers.
	 */
		for (j = 0; j < i; j++) {
			if (Vp[j] == va)
				break;
		}
		if (j < i)
			continue;
		if (Vp == NULL) {
			if ((Vp = (struct vnode **)malloc((MALLOC_S)
			    (sizeof(struct vnode *) * 10)))
			== NULL) {
				(void) fprintf(stderr,
				    "%s: no space for text vnode pointers\n",
				    Pn);
				exit(1);
			}
			Nv = 10;
		} else if (i >= Nv) {
			Nv += 10;
			if ((Vp = (struct vnode **)realloc((MALLOC_P *)Vp,
			    (MALLOC_S)(Nv * sizeof(struct vnode *))))
			== NULL) {
			    (void) fprintf(stderr,
				"%s: no more space for text vnode pointers\n",
				Pn);
			    exit(1);
			}
		}
		Vp[i++] = va;
	/*
	 * Allocate local file structure.
	 */
		switch (p.p_type) {
		case PT_DATA:
		case PT_TEXT:
			alloc_lfile(" txt", -1);
			break;
		case PT_MMAP:
			alloc_lfile(" mem", -1);
			break;
		default:
			(void) sprintf(fd, "R%02d", p.p_type);
			alloc_lfile(fd, -1);
		}
	/*
	 * Save vnode information.
	 */
		process_node((caddr_t)va);
		if (Lf->sf)
			link_lfile();
	}
}
#endif	/* HPUX8 */
