/*
 * dnode.c - DEC OSF/1 node functions for lsof
 */


/*
 * Copyright 1994 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: dnode.c,v 1.4 94/05/06 16:08:35 abe Exp $";
#endif


#include "lsof.h"


_PROTOTYPE(static int readvnode,(caddr_t va, struct vnode *v));


/*
 * process_node() - process vnode
 */

void
process_node(va)
	caddr_t va;			/* vnode kernel space address */
{
	struct advfsnode *a = NULL;
	struct cdnode *c = NULL;
	dev_t dev;
	char dev_ch[32];
	struct inode *i = NULL;
	struct vfs *la = NULL;
	struct mfsnode *m = NULL;
	struct procnode *p = NULL;
	struct procfsid *pfi;
	struct rnode *r = NULL;
	struct spec_node *s = NULL;
	struct s5inode *s5 = NULL;
	char *ty;
	enum vtype type;
	static struct vnode *v = NULL;
	struct l_vfs *vfs;
/*
 * Read the vnode.
 */
	if ( ! va) {
		enter_nm("no vnode address");
		return;
	}
	if (v == NULL) {

	/*
	 * Allocate space for the DEC Alpha OSF/1 vnode.
	 */
		if ((v = (struct vnode *)malloc(sizeof(struct vnode)-1+Vnmxp))
		== NULL) {
			(void) fprintf(stderr,
				"%s: no space for vnode buffer\n", Pn);
			exit(1);
		}
	}
	if (readvnode(va, v)) {
                enter_nm(Namech);
                return;
        }
/*
 * Get the mount structure and determine the vnode type.
 */
	if (v->v_mount == NULL || (vfs = readvfs(v->v_mount)) == NULL) {
		(void) sprintf(Namech, "bad mount for %#x at %#x", v,
			v->v_mount);
		enter_nm(Namech);
		return;
	}
	switch (vfs->type) {
	case MOUNT_NFS:
		Ntype = N_NFS;
		break;
	}
	if (Ntype == N_REGLR) {
		switch (v->v_type) {
		case VFIFO:
			Ntype = N_FIFO;
			break;
		}
	}
/*
 * Determine the lock type.
 */

	if (v->v_shlockc || v->v_exlockc) {
		if (v->v_shlockc && v->v_exlockc)
			Lf->lock = '*';
		else if (v->v_shlockc)
			Lf->lock = 'R';
		else
			Lf->lock = 'W';
	}
/*
 * Define the specific DEC Alpha OSF/1 node pointer.
 */
	switch (v->v_tag) {
	case VT_CDFS:
		c = (struct cdnode *)v->v_data;
		break;
	case VT_MFS:
		m = (struct mfsnode *)v->v_data;
		break;
	case VT_NFS:
		r = (struct rnode *)v->v_data;
		break;
	case VT_NON:
		s = (struct spec_node *)v->v_data;
		break;
	case VT_PRFS:
		p = (struct procnode *)v->v_data;
		break;
	case VT_S5FS:
		s5 = (struct s5inode *)v->v_data;
		break;
	case VT_MSFS:
		a = (struct advfsnode *)v->v_data;
		break;
	case VT_UFS:
		i = (struct inode *)v->v_data;
		break;
	}
/*
 * Get device and type for printing.
 */
	type = v->v_type;
	if (a != NULL)
		dev = (dev_t)vfs->fsid.val[0];
	else if (c != NULL)
		dev = c->cd_dev;
	else if (i != NULL)
		dev = (type == VCHR) ? i->i_din.di_db[0] : i->i_dev;
	else if (r != NULL)
		dev = r->r_attr.va_fsid;
	else if (s != NULL) {
		if (Clonedev != -1 && minor(s->sn_vattr.va_rdev) == 0)
		    dev = makedev(major(Clonedev), major(s->sn_vattr.va_rdev));
		else
		    dev = s->sn_vattr.va_rdev;
	} else if (s5 != NULL)
		dev = s5->i_dev;
/*
 * Obtain the inode number.
 */
	if (a != NULL)
		Lf->inode = a->a_number;
	else if (c != NULL)
		Lf->inode = c->cd_number;
	else if (i != NULL)
		Lf->inode = (long)i->i_number;
	else if (p != NULL && p->prc_pid != 0)
		Lf->inode = p->prc_pid + 64;
	else if (r != NULL)
		Lf->inode = r->r_attr.va_fileid;
	else if (s5 != NULL)
		Lf->inode = s5->i_number;
/*
 * Obtain the file size.
 */
	if (Foffset)
		Lf->off_def = 1;
	else {
		switch (Ntype) {
		case N_FIFO:
			if (!Fsize)
				Lf->off_def = 1;
			break;
		case N_NFS:
			if (r != NULL) {
				Lf->sz = r->r_attr.va_qsize;
				Lf->sz_def = 1;
			}
			break;
		case N_REGLR:
			if (type == VREG || type == VDIR) {
				if (a != NULL) {
					Lf->sz = a->a_size;
					Lf->sz_def = 1;
				} else if (i != NULL) {
					Lf->sz = i->i_din.di_qsize;
					Lf->sz_def = 1;
				} else if (m != NULL) {
					Lf->sz = m->mfs_size;
					Lf->sz_def = 1;
				} else if (s5 != NULL) {
					Lf->sz = s5->i_size;
					Lf->sz_def = 1;
				}
			} else if (type == VCHR && !Fsize)
				Lf->off_def = 1;
		}
	}
/*
 * Record an NFS file selection.
 */
	if (Ntype == N_NFS && Fnfs)
		Lf->sf |= SELNFS;
/*
 * Save the file system names.
 */
	if (vfs) {
		Lf->fsdir = vfs->dir;
		Lf->fsdev = vfs->fsname;
	}
/*
 * Format the vnode type, and possibly the device name.
 */
	switch (type) {

	case VNON:
		ty ="VNON";
		break;
	case VREG:
	case VDIR:
		ty = (type == VREG) ? "VREG" : "VDIR";
		Lf->dev_def = 1;
		Lf->dev = dev;
		break;
	case VBLK:
		ty = "VBLK";
		break;
	case VCHR:
		Lf->dev = dev;
		Lf->dev_def = Lf->is_chr_dev = 1;
		ty = "VCHR";
		break;
	case VLNK:
		ty = "VLNK";
		break;

#if	defined(VSOCK)
	case VSOCK:
		ty = "SOCK";
		break;
#endif

	case VBAD:
		ty = "VBAD";
		break;
	case VFIFO:
		Lf->dev = dev;
		Lf->dev_def = 1;
		ty = "FIFO";
		break;
	default:
		if (type > 9999)
			(void) sprintf(Lf->type, "*%03d", type % 1000);
		else
			(void) sprintf(Lf->type, "%4d", type);
		(void) strcpy(Namech, "unknown type");
		ty = NULL;
	}
	if (ty)
		(void) strcpy(Lf->type, ty);
/*
 * Handle some special cases:
 *
 * 	ioctl(fd, TIOCNOTTY) files;
 *	memory node files;
 *	/proc files.
 */

	if (type == VBAD)
		(void) strcpy(Namech, "(dead)");
	else if (m != NULL) {
		Lf->dev_def = 0;
		(void) sprintf(Namech, "%#x", m->mfs_baseoff);
		(void) strcpy(dev_ch, "    memory");
		enter_dev_ch(dev_ch);
	} else if (p != NULL) {
		Lf->dev_def = 0;
		if (p->prc_pid != 0)
			(void) sprintf(Namech, "/proc/%05d", p->prc_pid);
		else
			(void) strcpy(Namech, "/proc");
	}
/*
 * If this is a VCHR file and it's missing an inode number, try to
 * supply one.
 */
	if (Lf->inode == -1L && type == VCHR && Lf->dev_def)
		find_ch_ino();
/*
 * Test for specified file.
 */
	if (p != NULL) {
		if (Procsrch)
			Lf->sf |= SELNM;
		else if ((pfi = Procfsid) != NULL && p->prc_pid != 0) {
			for (; pfi; pfi = pfi->next) {
				if (pfi->pid == p->prc_pid) {
					Lf->sf |= SELNM;
					break;
				}
			}
		}
	}
	if (Sfile && is_file_named(NULL, type))
		Lf->sf |= SELNM;
/*
 * Enter name characters.
 */
	if (Namech[0])
		enter_nm(Namech);
}


/*
 * readvnode() - read vnode
 */

static int
readvnode(va, v)
	caddr_t va;			/* vnode kernel space address */
	struct vnode *v;		/* vnode buffer pointer */
{

	if (kread((KA_T)va, (char *)v, sizeof(struct vnode) - 1 + Vnmxp)) {
		(void) sprintf(Namech, "can't read vnode at %#x", va);
		return(1);
	}
	return(0);
}
