/*
 * dfile.c - SunOS (Solaris 1.x and 2.x) file processing functions for lsof
 */


/*
 * Copyright 1994 Purdue Research Foundation, West Lafayette, Indiana
 * 47907.  All rights reserved.
 *
 * Written by Victor A. Abell
 *
 * This software is not subject to any license of the American Telephone
 * and Telegraph Company or the Regents of the University of California.
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor Purdue University are responsible for any
 *    consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors and Purdue
 *    University must appear in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This notice may not be removed or altered.
 */

#ifndef lint
static char copyright[] =
"@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
static char *rcsid = "$Id: dfile.c,v 1.9 94/06/02 16:38:21 abe Exp $";
#endif


#include "lsof.h"


/*
 * is_file_named() - is this file named?
 */

int
is_file_named(p, nt, vt)
	char *p;			/* path name; NULL = search by device
					 * and inode (from *Lf) */
	int nt;				/* node type -- e.g., N_* */
	enum vtype vt;			/* vnode type */
{
	int f;
	struct sfile *s;
/*
 * Search the file chain.
 */
	for (f = 0, s = Sfile; s; s = s->next) {

	/*
	 * Search by path name, as requested.
	 */
		if (p != NULL) {
			if (strcmp(p, s->aname) == 0
			||  strcmp(p, s->name)  == 0)
			{
				f = 2;
				break;
			}
			continue;
		}
	/*
	 * Check for a Sun clone file.
	 */
		if (nt == N_STREAM && major(s->dev) == CLONEMAJ
		&& major(Lf->dev) == minor(s->dev)) {
			f = 1;
			break;
		}
	/*
	 * Check for a regular file or directory -- the device and
	 * inode numbers must match.
	 */
		if (s->type) {
			if (Lf->dev == s->dev && (ino_t)Lf->inode == s->i) {
				f = 1;
				break;
			}
			continue;
		}
	/*
	 * Check for a file system match.
	 * Try to avoid matching character files to non-character devices.
	 */
		if (Lf->dev == s->dev) {
			if ( ! (vt == VCHR && s->mode != S_IFCHR)) {
				f = 1;
				break;
			}
		}
	}
/*
 * Convert the name if a match occurred.
 */
	if (f) {
		if (f == 2)
			(void) strcpy(Namech, p);
		else {
			(void) strcpy(Namech, s->name);
			if (s->devnm)
				(void) sprintf(endnm(), " (%s)", s->devnm);
		}
		s->f = 1;
		return(1);
	}
	return(0);
}


/*
 * print_file() - print file
 */

void
print_file()
{
	if (!is_file_sel(Lf))
		return;
	if (Hdr == 0) {
	    (void) fputs("COMMAND     PID     USER   FD   TYPE     DEVICE ",
		stdout);
	    (void) printf("%10s", Foffset ? "OFFSET"
					  : Fsize ? "SIZE"
						  : "SIZE/OFF");
	    (void) puts("  INODE NAME");
	    Hdr++;
	}
	(void) printf("%-9.9s%6d %8.8s %4s%c%c %4.4s ",
			Lp->cmd,
			Lp->pid,
			printuid((UID_ARG)Lp->uid),
			Lf->fd,
			Lf->access,
			Lf->lock,
			Lf->type
		);
	if (Lf->dev_def) {

#if	defined(solaris)
	/*
	 * Avoid the Solaris major() and minor() functions from
	 * makedev(3C) to get printable major/minor numbers.
	 */
		(void) printf(" %4d,%4d ", (Lf->dev >> L_BITSMINOR) & L_MAXMAJ,
			Lf->dev & L_MAXMIN);
#else
		(void) printf(" %4d,%4d ", major(Lf->dev), minor(Lf->dev));
#endif

	} else {
		if (Lf->dev_ch) {
			(void) fputs(Lf->dev_ch, stdout);
			putchar(' ');
		} else
			(void) fputs("           ", stdout);
	}
	if (Lf->sz_def)
		(void) printf("%10u", Lf->sz);
	else if (Lf->off_def) {
		if (Lf->off)
			(void) printf("%#10x", (Lf->off & 0xffffffff));
		else
			(void) fputs("       0x0", stdout);
	} else
		    (void) fputs("          ", stdout);
	if (Lf->inode >= 0) {
		if (Lf->inode < 1000000l)
			(void) printf(" %6ld ", Lf->inode);
		else
			(void) printf(" *%05ld ", Lf->inode % 100000l);
	} else if (Lf->inode == -2l) {
		if (Lf->iproto[0] == '\0')
			(void) fputs("        ", stdout);
		else
			(void) printf(" %6.6s ", Lf->iproto);
	} else
		(void) fputs("        ", stdout);
	printname();
}


/*
 * process_file() - process file
 */

void
process_file(fp)
	struct file *fp;		/* kernel file structure address */
{
	struct file f;
	int flag;

	if (kread((KA_T)fp, (char *)&f, sizeof(f))) {
		(void) sprintf(Namech, "can't read file struct from %#x", fp);
		enter_nm(Namech);
		return;
	}
	Lf->off = f.f_offset;

	if (f.f_count) {

	/*
	 * Construct access code.
	 */
		if ((flag = (f.f_flag & (FREAD | FWRITE))) == FREAD)
			Lf->access = 'r';
		else if (flag == FWRITE)
			Lf->access = 'w';
		else if (flag == (FREAD | FWRITE))
			Lf->access = 'u';

#if	defined(solaris)
	/*
	 * Solaris file structures contain a vnode pointer.  Process it.
	 */
		process_node((caddr_t)f.f_vnode);
		return;
#else	/* !solaris */

	/*
	 * Process non-Solaris file structure by its type.
	 */
		switch (f.f_type) {

		case DTYPE_VNODE:
			process_node(f.f_data);
			return;
		case DTYPE_SOCKET:
			process_socket((caddr_t)f.f_data);
			return;
		default:
			if (f.f_type != 0 || f.f_ops != NULL) {
				(void) sprintf(Namech,
					"%#x file struct, ty=%#x, op=%#x",
					fp, f.f_type, f.f_ops);
				enter_nm(Namech);
				return;
			}
		}
#endif	/* solaris */

	}
	enter_nm("no more information");
}


/*
 * The ck_file_arg() function is obtained from ../common/ckfa.frag.
 */

/* CKFA_EXPDEV is not defined. */
/* CKFA_MPXCHAN is not defined. */
